@@ -19,20 +19,20 @@
 
 *****************************************************************************/
 
+#include "qsamplerAbout.h"
 #include "qsamplerChannel.h"
 
 #include "qsamplerMainForm.h"
 #include "qsamplerChannelForm.h"
 
-#include "config.h"
-
 #include <qfileinfo.h>
+#include <qcombobox.h>
 
 #ifdef CONFIG_LIBGIG
 #include "gig.h"
 #endif
 
-#define QSAMPLER_INSTRUMENT_MAX 8
+#define QSAMPLER_INSTRUMENT_MAX 100
 
 
 //-------------------------------------------------------------------------
@@ -57,7 +57,8 @@ qsamplerChannel::qsamplerChannel ( qsamplerMainForm *pMainForm, int iChannelID )
 	m_sAudioDriver      = "ALSA";
 	m_iAudioDevice      = -1;
 	m_fVolume           = 0.0;
-
+	m_bMute             = false;
+	m_bSolo             = false;
 }
 
 // Default destructor.
@@ -417,6 +418,94 @@ bool qsamplerChannel::setVolume ( float fVolume )
 }
 
 
+// Sampler channel mute state.
+bool qsamplerChannel::channelMute (void) const
+{
+	return m_bMute;
+}
+
+bool qsamplerChannel::setChannelMute ( bool bMute )
+{
+	if (client() == NULL || m_iChannelID < 0)
+		return false;
+	if (m_iInstrumentStatus == 100 && ((m_bMute && bMute) || (!m_bMute && !bMute)))
+		return true;
+
+#ifdef CONFIG_MUTE_SOLO
+	if (::lscp_set_channel_mute(client(), m_iChannelID, bMute) != LSCP_OK) {
+		appendMessagesClient("lscp_set_channel_mute");
+		return false;
+	}
+	appendMessages(QObject::tr("Mute: %1.").arg((int) bMute));
+	m_bMute = bMute;
+	return true;
+#else
+	return false;
+#endif
+}
+
+
+// Sampler channel solo state.
+bool qsamplerChannel::channelSolo (void) const
+{
+	return m_bSolo;
+}
+
+bool qsamplerChannel::setChannelSolo ( bool bSolo )
+{
+	if (client() == NULL || m_iChannelID < 0)
+		return false;
+	if (m_iInstrumentStatus == 100 && ((m_bSolo && bSolo) || (!m_bSolo && !bSolo)))
+		return true;
+
+#ifdef CONFIG_MUTE_SOLO
+	if (::lscp_set_channel_solo(client(), m_iChannelID, bSolo) != LSCP_OK) {
+		appendMessagesClient("lscp_set_channel_solo");
+		return false;
+	}
+	appendMessages(QObject::tr("Solo: %1.").arg((int) bSolo));
+	m_bSolo = bSolo;
+	return true;
+#else
+	return false;
+#endif
+}
+
+
+// Audio routing accessors.
+int qsamplerChannel::audioChannel ( int iAudioOut ) const
+{
+	return m_audioRouting[iAudioOut];
+}
+
+bool qsamplerChannel::setAudioChannel ( int iAudioOut, int iAudioIn )
+{
+	if (client() == NULL || m_iChannelID < 0)
+		return false;
+	if (m_iInstrumentStatus == 100 &&
+			m_audioRouting[iAudioOut] == iAudioIn)
+		return true;
+
+	if (::lscp_set_channel_audio_channel(client(),
+			m_iChannelID, iAudioOut, iAudioIn) != LSCP_OK) {
+		appendMessagesClient("lscp_set_channel_audio_channel");
+		return false;
+	}
+
+	appendMessages(QObject::tr("Audio Channel: %1 -> %2.")
+		.arg(iAudioOut).arg(iAudioIn));
+
+	m_audioRouting[iAudioOut] = iAudioIn;
+	return true;
+}
+
+// The audio routing map itself.
+const qsamplerChannelRoutingMap& qsamplerChannel::audioRouting (void) const
+{
+	return m_audioRouting;
+}
+
+
 // Istrument name remapper.
 void qsamplerChannel::updateInstrumentName (void)
 {
@@ -465,6 +554,10 @@ bool qsamplerChannel::updateChannelInfo (void)
 	m_iMidiChannel      = pChannelInfo->midi_channel;
 	m_iAudioDevice      = pChannelInfo->audio_device;
 	m_fVolume           = pChannelInfo->volume;
+#ifdef CONFIG_MUTE_SOLO
+	m_bMute             = pChannelInfo->mute;
+	m_bSolo             = pChannelInfo->solo;
+#endif
 	// Some sanity checks.
 	if (m_sEngineName == "NONE" || m_sEngineName.isEmpty())
 		m_sEngineName = QString::null;
@@ -493,6 +586,13 @@ bool qsamplerChannel::updateChannelInfo (void)
 		m_sMidiDriver = pDeviceInfo->driver;
 	}
 
+	// Set the audio routing map.
+	m_audioRouting.clear();
+	char **ppszRouting = pChannelInfo->audio_routing;
+	for (int i = 0; ppszRouting && ppszRouting[i]; i++) {
+		m_audioRouting[i] = ::atoi(ppszRouting[i]);
+	}
+
 	return true;
 }
 
@@ -602,7 +702,7 @@ QStringList qsamplerChannel::getInstrumentList( const QString& sInstrumentFile,
 	if (isInstrumentFile(sInstrumentFile)) {
 #ifdef CONFIG_LIBGIG
 		if (bInstrumentNames) {
-			RIFF::File *pRiff = new RIFF::File(sInstrumentFile);
+			RIFF::File *pRiff = new RIFF::File(sInstrumentFile.latin1());
 			gig::File  *pGig  = new gig::File(pRiff);
 			gig::Instrument *pInstrument = pGig->GetFirstInstrument();
 			while (pInstrument) {
@@ -633,7 +733,7 @@ QString qsamplerChannel::getInstrumentName( const QString& sInstrumentFile,
 		sInstrumentName = QFileInfo(sInstrumentFile).fileName();
 #ifdef CONFIG_LIBGIG
 		if (bInstrumentNames) {
-			RIFF::File *pRiff = new RIFF::File(sInstrumentFile);
+			RIFF::File *pRiff = new RIFF::File(sInstrumentFile.latin1());
 			gig::File  *pGig  = new gig::File(pRiff);
 			int iIndex = 0;
 			gig::Instrument *pInstrument = pGig->GetFirstInstrument();
@@ -674,4 +774,157 @@ QString qsamplerChannel::loadingInstrument (void) {
 }
 
 
+
+//-------------------------------------------------------------------------
+// qsamplerChannelRoutingTable - Channel routing table.
+//
+
+// Constructor.
+qsamplerChannelRoutingTable::qsamplerChannelRoutingTable (
+	QWidget *pParent, const char *pszName )
+	: QTable(pParent, pszName)
+{
+	// Set fixed number of columns.
+	QTable::setNumCols(2);
+	QTable::setShowGrid(false);
+	QTable::setSorting(false);
+	QTable::setFocusStyle(QTable::FollowStyle);
+	QTable::setSelectionMode(QTable::NoSelection);
+	// No vertical header.
+	QTable::verticalHeader()->hide();
+	QTable::setLeftMargin(0);
+	// Initialize the fixed table column headings.
+	QHeader *pHeader = QTable::horizontalHeader();
+	pHeader->setLabel(0, tr("Sampler Channel"));
+	pHeader->setLabel(1, tr("Device Channel"));
+	// Set read-onlyness of each column
+	QTable::setColumnReadOnly(0, true);
+//	QTable::setColumnReadOnly(1, false); -- of course not.
+	QTable::setColumnStretchable(1, true);
+}
+
+// Default destructor.
+qsamplerChannelRoutingTable::~qsamplerChannelRoutingTable (void)
+{
+}
+
+
+// Routing map table renderer.
+void qsamplerChannelRoutingTable::refresh ( qsamplerDevice *pDevice,
+	const qsamplerChannelRoutingMap& routing )
+{
+	if (pDevice == NULL)
+		return;
+
+	// Always (re)start it empty.
+	QTable::setUpdatesEnabled(false);
+	QTable::setNumRows(0);
+
+	// The common device port item list.
+	QStringList opts;
+	qsamplerDevicePortList& ports = pDevice->ports();
+	qsamplerDevicePort *pPort;
+	for (pPort = ports.first(); pPort; pPort = ports.next()) {
+		opts.append(pDevice->deviceTypeName()
+			+ ' ' + pDevice->driverName()
+			+ ' ' + pPort->portName());
+	}
+
+	// Those items shall have a proper pixmap...
+	QPixmap pmChannel = QPixmap::fromMimeSource("qsamplerChannel.png");
+	QPixmap pmDevice;
+	switch (pDevice->deviceType()) {
+	case qsamplerDevice::Audio:
+		pmDevice = QPixmap::fromMimeSource("audio2.png");
+		break;
+	case qsamplerDevice::Midi:
+		pmDevice = QPixmap::fromMimeSource("midi2.png");
+		break;
+	case qsamplerDevice::None:
+		break;
+	}
+
+	// Fill the routing table...
+	QTable::insertRows(0, routing.count());
+	int iRow = 0;
+	qsamplerChannelRoutingMap::ConstIterator iter;
+	for (iter = routing.begin(); iter != routing.end(); ++iter) {
+		QTable::setPixmap(iRow, 0, pmChannel);
+		QTable::setText(iRow, 0, pDevice->deviceTypeName()
+			+ ' ' + QString::number(iter.key()));
+		qsamplerChannelRoutingComboBox *pComboItem =
+			new qsamplerChannelRoutingComboBox(this, opts, pmDevice);
+		pComboItem->setCurrentItem(iter.data());
+		QTable::setItem(iRow, 1, pComboItem);
+		++iRow;
+	}
+
+	// Adjust optimal column widths.
+	QTable::adjustColumn(0);
+	QTable::adjustColumn(1);
+
+	QTable::setUpdatesEnabled(true);
+	QTable::updateContents();
+}
+
+
+// Commit any pending editing.
+void qsamplerChannelRoutingTable::flush (void)
+{
+	if (QTable::isEditing())
+	    QTable::endEdit(QTable::currEditRow(), QTable::currEditCol(), true, true);
+}
+
+
+//-------------------------------------------------------------------------
+// qsamplerChannelRoutingComboBox - Custom combo box for routing table.
+//
+
+// Constructor.
+qsamplerChannelRoutingComboBox::qsamplerChannelRoutingComboBox (
+	QTable *pTable, const QStringList& list, const QPixmap& pixmap )
+	: QTableItem(pTable, QTableItem::WhenCurrent, QString::null, pixmap),
+	m_list(list)
+{
+	m_iCurrentItem = 0;
+}
+
+// Public accessors.
+void qsamplerChannelRoutingComboBox::setCurrentItem ( int iCurrentItem )
+{
+	m_iCurrentItem = iCurrentItem;
+
+	QTableItem::setText(m_list[iCurrentItem]);
+}
+
+int qsamplerChannelRoutingComboBox::currentItem (void) const
+{
+	return m_iCurrentItem;
+}
+
+// Virtual implemetations.
+QWidget *qsamplerChannelRoutingComboBox::createEditor (void) const
+{
+	QComboBox *pComboBox = new QComboBox(QTableItem::table()->viewport());
+	QObject::connect(pComboBox, SIGNAL(activated(int)),
+		QTableItem::table(), SLOT(doValueChanged()));
+	for (QStringList::ConstIterator iter = m_list.begin();
+			iter != m_list.end(); iter++) {
+		pComboBox->insertItem(QTableItem::pixmap(), *iter);
+	}
+	pComboBox->setCurrentItem(m_iCurrentItem);
+	return pComboBox;
+}
+
+void qsamplerChannelRoutingComboBox::setContentFromEditor ( QWidget *pWidget )
+{
+	if (pWidget->inherits("QComboBox")) {
+		QComboBox *pComboBox = (QComboBox *) pWidget;
+		m_iCurrentItem = pComboBox->currentItem();
+		QTableItem::setText(pComboBox->currentText());
+	}
+	else QTableItem::setContentFromEditor(pWidget);
+}
+
+
 // end of qsamplerChannel.cpp
