@@ -22,16 +22,21 @@
 #ifndef __qsamplerChannel_h
 #define __qsamplerChannel_h
 
-#include <qobject.h>
+#include <qtable.h>
 
 #include <lscp/client.h>
 #include <lscp/device.h>
 
 #include "qsamplerOptions.h"
 
+class qsamplerDevice;
 class qsamplerMainForm;
 
 
+// Typedef'd QMap.
+typedef QMap<int, int> qsamplerChannelRoutingMap;
+
+
 //-------------------------------------------------------------------------
 // qsamplerChannel - Sampler channel structure.
 //
@@ -108,6 +113,20 @@ public:
 	float    volume() const;
 	bool     setVolume(float fVolume);
 
+	// Sampler channel mute state.
+	bool     channelMute() const;
+	bool     setChannelMute(bool bMute);
+
+	// Sampler channel solo state.
+	bool     channelSolo() const;
+	bool     setChannelSolo(bool bSolo);
+
+	// Audio routing accessors.
+	int      audioChannel(int iAudioOut) const;
+	bool     setAudioChannel(int iAudioOut, int iAudioIn);
+	// The audio routing map itself.
+	const qsamplerChannelRoutingMap& audioRouting() const;
+
 	// Istrument name remapper.
 	void     updateInstrumentName();
 
@@ -165,8 +184,68 @@ private:
 	QString m_sAudioDriver;
 	int     m_iAudioDevice;
 	float   m_fVolume;
+	bool    m_bMute;
+	bool    m_bSolo;
+
+	// The audio routing mapping.
+	qsamplerChannelRoutingMap m_audioRouting;
 };
 
+
+//-------------------------------------------------------------------------
+// qsamplerChannelRoutingTable - Channel routing table widget.
+//
+
+class qsamplerChannelRoutingTable : public QTable
+{
+	Q_OBJECT
+
+public:
+
+	// Constructor.
+	qsamplerChannelRoutingTable(QWidget *pParent = 0, const char *pszName = 0);
+	// Default destructor.
+	~qsamplerChannelRoutingTable();
+
+	// Common parameter table renderer.
+	void refresh(qsamplerDevice *pDevice,
+		const qsamplerChannelRoutingMap& routing);
+
+	// Commit any pending editing.
+	void flush();
+};
+
+
+//-------------------------------------------------------------------------
+// qsamplerChannelRoutingComboBox - Custom combo box for routing table.
+//
+
+class qsamplerChannelRoutingComboBox : public QTableItem
+{
+public:
+
+	// Constructor.
+	qsamplerChannelRoutingComboBox(QTable *pTable,
+		const QStringList& list, const QPixmap& pixmap);
+
+	// Public accessors.
+	void setCurrentItem(int iCurrentItem);
+	int currentItem() const;
+
+protected:
+
+	// Virtual implemetations.
+	QWidget *createEditor() const;
+	void setContentFromEditor(QWidget *pWidget);
+
+private:
+
+	// Initial value holders
+	QStringList m_list;
+	int m_iCurrentItem;
+};
+
+
 #endif  // __qsamplerChannel_h
 
 
