@@ -20,16 +20,15 @@
 
 *****************************************************************************/
 
+#include "qsamplerAbout.h"
+#include "qsamplerDeviceForm.h"
+
 #include <qvalidator.h>
 #include <qmessagebox.h>
 #include <qfiledialog.h>
 #include <qfileinfo.h>
 #include <qlistbox.h>
 
-#include "qsamplerDeviceForm.h"
-
-#include "config.h"
-
 
 // Kind of constructor.
 void qsamplerChannelForm::init (void)
@@ -72,7 +71,7 @@ void qsamplerChannelForm::setup ( qsamplerChannel *pChannel )
 
 	// It can be a brand new channel, remember?
 	bool bNew = (m_pChannel->channelID() < 0);
-	setCaption(m_pChannel->channelName());
+	setCaption(QSAMPLER_TITLE ": " + m_pChannel->channelName());
 
 	// Check if we're up and connected.
 	if (m_pChannel->client() == NULL)
@@ -88,6 +87,10 @@ void qsamplerChannelForm::setup ( qsamplerChannel *pChannel )
 	// Load combo box history...
 	pOptions->loadComboBoxHistory(InstrumentFileComboBox);
 
+	// Notify.that we've just changed one audio route.
+	QObject::connect(AudioRoutingTable, SIGNAL(valueChanged(int,int)),
+		this, SLOT(changeAudioRouting(int,int)));
+
 	// Populate Engines list.
 	const char **ppszEngines = ::lscp_list_available_engines(m_pChannel->client());
 	if (ppszEngines) {
@@ -180,9 +183,9 @@ void qsamplerChannelForm::setup ( qsamplerChannel *pChannel )
 
 	// As convenient, make it ready on stabilizeForm() for
 	// prompt acceptance, if we got the minimum required...
-	if (sEngineName != qsamplerChannel::noEngineName() &&
+/*	if (sEngineName != qsamplerChannel::noEngineName() &&
 		sInstrumentFile != qsamplerChannel::noInstrumentName())
-		m_iDirtyCount++;
+		m_iDirtyCount++; */
 	// Done.
 	m_iDirtySetup--;
 	stabilizeForm();
@@ -199,6 +202,9 @@ void qsamplerChannelForm::accept (void)
 	if (pOptions == NULL)
 		return;
 
+	// Flush any pending editing...
+	AudioRoutingTable->flush();
+
 	// We'll go for it!
 	if (m_iDirtyCount > 0) {
 		int iErrors = 0;
@@ -215,6 +221,15 @@ void qsamplerChannelForm::accept (void)
 				iErrors++;
 			else if (!m_pChannel->setAudioDevice(pDevice->deviceID()))
 				iErrors++;
+			else if (!m_audioRouting.isEmpty()) {
+				// Set the audio route changes...
+				qsamplerChannelRoutingMap::ConstIterator iter;
+				for (iter = m_audioRouting.begin();
+						iter != m_audioRouting.end(); ++iter) {
+					if (!m_pChannel->setAudioChannel(iter.key(), iter.data()))
+						iErrors++;
+				}
+			}
 		}
 		// Accept MIDI driver or device selection...
 		if (m_midiDevices.isEmpty()) {
@@ -263,7 +278,8 @@ void qsamplerChannelForm::reject (void)
 
 	// Check if there's any pending changes...
 	if (m_iDirtyCount > 0 && OkPushButton->isEnabled()) {
-		switch (QMessageBox::warning(this, tr("Warning"),
+		switch (QMessageBox::warning(this,
+			QSAMPLER_TITLE ": " + tr("Warning"),
 			tr("Some channel settings have been changed.\n\n"
 			"Do you want to apply the changes?"),
 			tr("Apply"), tr("Discard"), tr("Cancel"))) {
@@ -292,10 +308,10 @@ void qsamplerChannelForm::openInstrumentFile (void)
 	// FIXME: the instrument file filters should be restricted,
 	// depending on the current engine.
 	QString sInstrumentFile = QFileDialog::getOpenFileName(
-			pOptions->sInstrumentDir,                   // Start here.
-			tr("Instrument files") + " (*.gig *.dls)",  // Filter (GIG and DLS files)
-			this, 0,                                    // Parent and name (none)
-			tr("Instrument files")                      // Caption.
+		pOptions->sInstrumentDir,                   // Start here.
+		tr("Instrument files") + " (*.gig *.dls)",  // Filter (GIG and DLS files)
+		this, 0,                                    // Parent and name (none)
+		QSAMPLER_TITLE ": " + tr("Instrument files")	// Caption.
 	);
 
 	if (sInstrumentFile.isEmpty())
@@ -356,7 +372,7 @@ void qsamplerChannelForm::selectMidiDriverItem ( const QString& sMidiDriver )
 	const QString sDriverName = sMidiDriver.upper();
 
 	// Save current device id.
-	int iDeviceID = -1;
+	int iDeviceID = 0;
 	qsamplerDevice *pDevice = m_midiDevices.at(MidiDeviceComboBox->currentItem());
 	if (pDevice)
 		iDeviceID = pDevice->deviceID();
@@ -382,10 +398,7 @@ void qsamplerChannelForm::selectMidiDriverItem ( const QString& sMidiDriver )
 
 	// Do proper enabling...
 	bool bEnabled = !m_midiDevices.isEmpty();
-	if (!bEnabled) {
-		MidiDeviceComboBox->insertItem(
-			tr("(New MIDI %1 device)").arg(sMidiDriver));
-	} else if (iDeviceID >= 0) {
+	if (bEnabled) {
 		// Select the previous current device...
 		int iMidiItem = 0;
 		for (pDevice = m_midiDevices.first();
@@ -393,11 +406,14 @@ void qsamplerChannelForm::selectMidiDriverItem ( const QString& sMidiDriver )
 					pDevice = m_midiDevices.next()) {
 			if (pDevice->deviceID() == iDeviceID) {
 				MidiDeviceComboBox->setCurrentItem(iMidiItem);
-			//	selectMidiDeviceItem(iMidiItem);
+				selectMidiDeviceItem(iMidiItem);
 				break;
 			}
 			iMidiItem++;
 		}
+	} else {
+		MidiDeviceComboBox->insertItem(
+			tr("(New MIDI %1 device)").arg(sMidiDriver));
 	}
 	MidiDeviceTextLabel->setEnabled(bEnabled);
 	MidiDeviceComboBox->setEnabled(bEnabled);
@@ -455,7 +471,7 @@ void qsamplerChannelForm::selectAudioDriverItem ( const QString& sAudioDriver )
 	const QString sDriverName = sAudioDriver.upper();
 
 	// Save current device id.
-	int iDeviceID = -1;
+	int iDeviceID = 0;
 	qsamplerDevice *pDevice = m_audioDevices.at(AudioDeviceComboBox->currentItem());
 	if (pDevice)
 		iDeviceID = pDevice->deviceID();
@@ -481,10 +497,7 @@ void qsamplerChannelForm::selectAudioDriverItem ( const QString& sAudioDriver )
 
 	// Do proper enabling...
 	bool bEnabled = !m_audioDevices.isEmpty();
-	if (!bEnabled) {
-		AudioDeviceComboBox->insertItem(
-			tr("(New Audio %1 device)").arg(sAudioDriver));
-	} else if (iDeviceID >= 0) {
+	if (bEnabled) {
 		// Select the previous current device...
 		int iAudioItem = 0;
 		for (pDevice = m_audioDevices.first();
@@ -492,14 +505,19 @@ void qsamplerChannelForm::selectAudioDriverItem ( const QString& sAudioDriver )
 					pDevice = m_audioDevices.next()) {
 			if (pDevice->deviceID() == iDeviceID) {
 				AudioDeviceComboBox->setCurrentItem(iAudioItem);
-			//	selectAudioDeviceItem(iAudioItem);
+				selectAudioDeviceItem(iAudioItem);
 				break;
 			}
 			iAudioItem++;
 		}
+	} else {
+		AudioDeviceComboBox->insertItem(
+			tr("(New Audio %1 device)").arg(sAudioDriver));
+		AudioRoutingTable->setNumRows(0);
 	}
 	AudioDeviceTextLabel->setEnabled(bEnabled);
 	AudioDeviceComboBox->setEnabled(bEnabled);
+	AudioRoutingTable->setEnabled(bEnabled);
 }
 
 
@@ -519,7 +537,10 @@ void qsamplerChannelForm::selectAudioDeviceItem ( int iAudioItem )
 {
 	qsamplerDevice *pDevice = m_audioDevices.at(iAudioItem);
 	if (pDevice) {
-		// Is there anything to do here?
+		// Refresh the audio routing table.
+		AudioRoutingTable->refresh(pDevice, m_pChannel->audioRouting());
+		// Reset routing change map.
+		m_audioRouting.clear();
 	}
 }
 
@@ -543,6 +564,29 @@ void qsamplerChannelForm::setupAudioDevice (void)
 }
 
 
+// Audio routing change slot.
+void qsamplerChannelForm::changeAudioRouting ( int iRow, int iCol )
+{
+	if (m_iDirtySetup > 0)
+		return;
+	if (iRow < 0 || iCol < 0)
+		return;
+
+	// Verify that this is a QComboTableItem (magic rtti == 1)
+	QTableItem *pItem = AudioRoutingTable->item(iRow, iCol);
+	if (pItem == NULL)
+		return;
+	qsamplerChannelRoutingComboBox *pComboItem =
+		static_cast<qsamplerChannelRoutingComboBox*> (pItem);
+	// FIXME: Its not garanteed that we must have
+	// iAudioOut == iRow on all times forth!
+	m_audioRouting[iRow] = pComboItem->currentItem();
+
+	// And let's get dirty...
+	optionsChanged();
+}
+
+
 // UPdate all device lists slot.
 void qsamplerChannelForm::updateDevices (void)
 {
