@@ -45,8 +45,6 @@
 #include "qsamplerDeviceForm.h"
 #include "qsamplerOptionsForm.h"
 
-#include "config.h"
-
 #ifdef HAVE_SIGNAL_H
 #include <signal.h>
 #endif
@@ -473,10 +471,10 @@ bool qsamplerMainForm::openSession (void)
 
     // Ask for the filename to open...
     QString sFilename = QFileDialog::getOpenFileName(
-            m_pOptions->sSessionDir,                // Start here.
-            tr("LSCP Session files") + " (*.lscp)", // Filter (LSCP files)
-            this, 0,                                // Parent and name (none)
-            tr("Open Session")                      // Caption.
+		m_pOptions->sSessionDir,                // Start here.
+		tr("LSCP Session files") + " (*.lscp)", // Filter (LSCP files)
+		this, 0,                                // Parent and name (none)
+		QSAMPLER_TITLE ": " + tr("Open Session")	// Caption.
     );
 
     // Have we cancelled?
@@ -507,10 +505,10 @@ bool qsamplerMainForm::saveSession ( bool bPrompt )
             sFilename = m_pOptions->sSessionDir;
         // Prompt the guy...
         sFilename = QFileDialog::getSaveFileName(
-                sFilename,                              // Start here.
-                tr("LSCP Session files") + " (*.lscp)", // Filter (LSCP files)
-                this, 0,                                // Parent and name (none)
-                tr("Save Session")                      // Caption.
+			sFilename,                              // Start here.
+			tr("LSCP Session files") + " (*.lscp)", // Filter (LSCP files)
+			this, 0,                                // Parent and name (none)
+			QSAMPLER_TITLE ": " + tr("Save Session")	// Caption.
         );
         // Have we cancelled it?
         if (sFilename.isEmpty())
@@ -520,7 +518,8 @@ bool qsamplerMainForm::saveSession ( bool bPrompt )
             sFilename += ".lscp";
         // Check if already exists...
         if (sFilename != m_sFilename && QFileInfo(sFilename).exists()) {
-            if (QMessageBox::warning(this, tr("Warning"),
+            if (QMessageBox::warning(this,
+				QSAMPLER_TITLE ": " + tr("Warning"),
                 tr("The file already exists:\n\n"
                    "\"%1\"\n\n"
                    "Do you want to replace it?")
@@ -542,7 +541,8 @@ bool qsamplerMainForm::closeSession ( bool bForce )
 
     // Are we dirty enough to prompt it?
     if (m_iDirtyCount > 0) {
-        switch (QMessageBox::warning(this, tr("Warning"),
+        switch (QMessageBox::warning(this,
+			QSAMPLER_TITLE ": " + tr("Warning"),
             tr("The current session has been changed:\n\n"
             "\"%1\"\n\n"
             "Do you want to save the changes?")
@@ -600,7 +600,7 @@ bool qsamplerMainForm::loadSessionFile ( const QString& sFilename )
     QTextStream ts(&file);
     while (!ts.atEnd()) {
         // Read the line.
-        QString sCommand = ts.readLine().simplifyWhiteSpace();
+        QString sCommand = ts.readLine().stripWhiteSpace();
         // If not empty, nor a comment, call the server...
         if (!sCommand.isEmpty() && sCommand[0] != '#') {
             appendMessagesColor(sCommand, "#996633");
@@ -787,7 +787,19 @@ bool qsamplerMainForm::saveSessionFile ( const QString& sFilename )
                 ts << endl;
                 ts << "LOAD ENGINE " << pChannel->engineName() << " " << iChannel << endl;
                 ts << "LOAD INSTRUMENT NON_MODAL '" << pChannel->instrumentFile() << "' " << pChannel->instrumentNr() << " " << iChannel << endl;
+				qsamplerChannelRoutingMap::ConstIterator audioRoute;
+				for (audioRoute = pChannel->audioRouting().begin();
+						audioRoute != pChannel->audioRouting().end();
+							++audioRoute) {
+					ts << "SET CHANNEL AUDIO_OUTPUT_CHANNEL " << iChannel
+						<< " " << audioRoute.key()
+						<< " " << audioRoute.data() << endl;
+				}
                 ts << "SET CHANNEL VOLUME " << iChannel << " " << pChannel->volume() << endl;
+				if (pChannel->channelMute())
+					ts << "SET CHANNEL MUTE " << iChannel << " 1" << endl;
+				if (pChannel->channelSolo())
+					ts << "SET CHANNEL SOLO " << iChannel << " 1" << endl;
                 ts << endl;
             }
         }
@@ -879,7 +891,8 @@ void qsamplerMainForm::fileReset (void)
         return;
 
     // Ask user whether he/she want's an internal sampler reset...
-    if (QMessageBox::warning(this, tr("Warning"),
+    if (QMessageBox::warning(this,
+		QSAMPLER_TITLE ": " + tr("Warning"),
         tr("Resetting the sampler instance will close\n"
            "all device and channel configurations.\n\n"
            "Please note that this operation may cause\n"
@@ -914,7 +927,8 @@ void qsamplerMainForm::fileRestart (void)
     // Ask user whether he/she want's a complete restart...
     // (if we're currently up and running)
     if (bRestart && m_pClient) {
-        bRestart = (QMessageBox::warning(this, tr("Warning"),
+        bRestart = (QMessageBox::warning(this,
+			QSAMPLER_TITLE ": " + tr("Warning"),
             tr("New settings will be effective after\n"
                "restarting the client/server connection.\n\n"
                "Please note that this operation may cause\n"
@@ -990,7 +1004,8 @@ void qsamplerMainForm::editRemoveChannel (void)
 
     // Prompt user if he/she's sure about this...
     if (m_pOptions && m_pOptions->bConfirmRemove) {
-        if (QMessageBox::warning(this, tr("Warning"),
+        if (QMessageBox::warning(this,
+			QSAMPLER_TITLE ": " + tr("Warning"),
             tr("About to remove channel:\n\n"
                "%1\n\n"
                "Are you sure?")
@@ -1173,7 +1188,8 @@ void qsamplerMainForm::viewOptions (void)
                 (!bOldStdoutCapture &&  m_pOptions->bStdoutCapture) ||
                 ( bOldKeepOnTop     && !m_pOptions->bKeepOnTop)     ||
                 (!bOldKeepOnTop     &&  m_pOptions->bKeepOnTop)) {
-                QMessageBox::information(this, tr("Information"),
+                QMessageBox::information(this,
+					QSAMPLER_TITLE ": " + tr("Information"),
                     tr("Some settings may be only effective\n"
                        "next time you start this program."), tr("OK"));
                 updateMessagesCapture();
@@ -1300,6 +1316,11 @@ void qsamplerMainForm::helpAbout (void)
     sText += "<small><font color=\"red\">";
     sText += tr("LSCP (liblscp) instrument_name support disabled.");
     sText += "</font></small><br />";
+#endif
+#ifndef CONFIG_MUTE_SOLO
+    sText += "<small><font color=\"red\">";
+    sText += tr("Sampler channel Mute/Solo support disabled.");
+    sText += "</font></small><br />";
 #endif
     sText += "<br />\n";
     sText += tr("Using") + ": ";
@@ -1308,9 +1329,9 @@ void qsamplerMainForm::helpAbout (void)
     sText += ::lscp_client_version();
 #ifdef CONFIG_LIBGIG
     sText += ", ";
-    sText += gig::libraryName();
+    sText += gig::libraryName().c_str();
     sText += " ";
-    sText += gig::libraryVersion();
+    sText += gig::libraryVersion().c_str();
 #endif
     sText += "<br />\n";
     sText += "<br />\n";
@@ -1336,7 +1357,7 @@ void qsamplerMainForm::stabilizeForm (void)
     // Update the main application caption...
     QString sSessionName = sessionName(m_sFilename);
     if (m_iDirtyCount > 0)
-        sSessionName += '*';
+        sSessionName += " *";
     setCaption(tr(QSAMPLER_TITLE " - [%1]").arg(sSessionName));
 
     // Update the main menu state...
@@ -1600,7 +1621,8 @@ void qsamplerMainForm::appendMessagesError( const QString& s )
 
     appendMessagesColor(s.simplifyWhiteSpace(), "#ff0000");
 
-    QMessageBox::critical(this, tr("Error"), s, tr("Cancel"));
+    QMessageBox::critical(this,
+		QSAMPLER_TITLE ": " + tr("Error"), s, tr("Cancel"));
 }
 
 
@@ -1872,7 +1894,8 @@ void qsamplerMainForm::startServer (void)
 
     // Is the server process instance still here?
     if (m_pServer) {
-        switch (QMessageBox::warning(this, tr("Warning"),
+        switch (QMessageBox::warning(this,
+			QSAMPLER_TITLE ": " + tr("Warning"),
             tr("Could not start the LinuxSampler server.\n\n"
                "Maybe it ss already started."),
             tr("Stop"), tr("Kill"), tr("Cancel"))) {
